﻿Imports System.Globalization

Public Class Form1
    ' Compare plans.
    Private Sub btnGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGo.Click
        Dim annual_contribution As Decimal = _
            Decimal.Parse(txtAnnualContribution.Text, NumberStyles.Any)

        Dim tax_rate As Decimal = _
            Decimal.Parse(txtTaxRate.Text.Replace("%", ""), _
                NumberStyles.Any) / 100
        If (tax_rate >= 1) Then tax_rate /= 100

        Dim interest_rate As Decimal = _
            Decimal.Parse(txtInterestRate.Text.Replace("%", ""), _
                NumberStyles.Any) / 100
        If (interest_rate >= 1) Then interest_rate /= 100

        Dim num_years As Integer = CInt(nudYears.Value)

        Dim balance_bank As Decimal = 0
        Dim balance_401k As Decimal = 0
        Dim balance_roth As Decimal = 0

        ' Generate the data points.
        Dim pt_bank(0 To num_years * 2 - 1) As PointF
        Dim pt_401k(0 To num_years) As PointF
        Dim pt_roth(0 To num_years) As PointF
        For year As Integer = 0 To num_years - 1
            ' Bank balance += interest + contribution, ...
            Dim interest As Decimal = balance_bank * interest_rate
            balance_bank += interest + annual_contribution
            pt_bank(2 * year) = New PointF(year, CSng(balance_bank))

            ' ... minus taxes on the interest and contribution.
            balance_bank -= tax_rate * (interest + annual_contribution)
            pt_bank(2 * year + 1) = New PointF(year, CSng(balance_bank))

            ' 401(k) balance += interest + contribution.
            balance_401k += (balance_401k * interest_rate + annual_contribution)
            pt_401k(year) = New PointF(year, CSng(balance_401k))

            ' Roth balance += interest + contribution - taxes on contribution.
            balance_roth += balance_roth * interest_rate + _
                annual_contribution * (1 - tax_rate)
            pt_roth(year) = New PointF(year, CSng(balance_roth))
        Next year

        ' Display the final results.
        txtFinalSavings.Text = balance_bank.ToString("c")
        txtFinal401k.Text = balance_401k.ToString("c")
        txtFinalRoth.Text = balance_roth.ToString("c")

        ' Add the final point after removing taxes from the 401(k).
        Dim penalty As Decimal = _
            Decimal.Parse(txtPenalty.Text.Replace("%", ""), _
                NumberStyles.Any) / 100
        If (penalty >= 1) Then penalty /= 100

        Dim tax_at_end As Decimal = _
            Decimal.Parse(txtTaxAtEnd.Text.Replace("%", ""), _
                NumberStyles.Any) / 100
        If (tax_at_end >= 1) Then tax_at_end /= 100

        balance_401k *= 1 - (penalty + tax_at_end)
        pt_401k(num_years) = New PointF(num_years - 1, CSng(balance_401k))

        balance_roth *= 1 - penalty
        pt_roth(num_years) = New PointF(num_years - 1, CSng(balance_roth))

        ' Display the final surrender values.
        txt401kSurrendered.Text = balance_401k.ToString("c")
        txtRothSurrendered.Text = balance_roth.ToString("c")

        ' Plot the results.
        ' Find a reasonable scale.
        Dim scale_x As Single = CSng(picGraph.ClientSize.Width / num_years)
        Dim scale_y As Single = picGraph.ClientSize.Height / pt_401k(num_years - 1).Y

        Dim bm As New Bitmap(picGraph.ClientSize.Width, picGraph.ClientSize.Height)
        Using gr As Graphics = Graphics.FromImage(bm)
            ' Scale the image.
            gr.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias
            gr.ScaleTransform(scale_x, -scale_y)

            ' Translate the image so it shows.
            gr.TranslateTransform(0, picGraph.ClientSize.Height, _
                System.Drawing.Drawing2D.MatrixOrder.Append)

            ' Draw the curves.
            gr.DrawLines(Pens.Red, pt_bank)
            gr.DrawLines(Pens.Blue, pt_401k)
            gr.DrawLines(Pens.Green, pt_roth)

            ' Transform the final points.
            Dim transformed_points(0 To 2) As PointF
            transformed_points(0) = pt_bank(2 * num_years - 1)
            transformed_points(1) = pt_401k(num_years)
            transformed_points(2) = pt_roth(num_years)
            gr.Transform.TransformPoints(transformed_points)

            ' Draw the final values.
            gr.ResetTransform()
            gr.FillEllipse(Brushes.Red, _
                transformed_points(0).X - 2, _
                transformed_points(0).Y - 2, _
                5, 5)
            gr.FillEllipse(Brushes.Blue, _
                transformed_points(1).X - 2, _
                transformed_points(1).Y - 2, _
                5, 5)
            gr.FillEllipse(Brushes.Green, _
                transformed_points(2).X - 2, _
                transformed_points(2).Y - 2, _
                5, 5)
        End Using

        ' Display the result.
        picGraph.Image = bm
    End Sub
End Class
